/********************************************************************
 Software License Agreement:

 The software supplied herewith by Microchip Technology Incorporated
 (the "Company") for its PIC(R) Microcontroller is intended and
 supplied to you, the Company's customer, for use solely and
 exclusively on Microchip PIC Microcontroller products. The
 software is owned by the Company and/or its supplier, and is
 protected under applicable copyright laws. All rights are reserved.
 Any use in violation of the foregoing restrictions may subject the
 user to criminal sanctions under applicable laws, as well as to
 civil liability for the breach of the terms and conditions of this
 license.

 THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *******************************************************************/

/** INCLUDES *******************************************************/
#include "system.h"

#include <stdint.h>
#include <string.h>
#include <stddef.h>

#include "usb.h"

//#include <app_led_usb_status.h>
#include "app_device_cdc_basic.h"
#include "usb_config.h"

#include <stdlib.h>
#include <stdio.h>
//#include <string.h>

/** VARIABLES ******************************************************/

//static bool buttonPressed;
static char buttonMessage[] = "Button pressed.\r\n";
static uint8_t readBuffer[CDC_DATA_OUT_EP_SIZE];//=64
static uint8_t writeBuffer[CDC_DATA_IN_EP_SIZE];//=64

#define _XTAL_FREQ  48000000             //__delay_ms





/*********************************************************************
* Function: void APP_DeviceCDCBasicDemoInitialize(void);
*
* Overview: Initializes the demo code
*
* PreCondition: None
*
* Input: None
*
* Output: None
*
********************************************************************/
void APP_DeviceCDCBasicDemoInitialize()
{
    CDCInitEP();

    
    line_coding.bCharFormat = 0;
    line_coding.bDataBits = 8;
    line_coding.bParityType = 0;
    line_coding.dwDTERate = 9600;

    //buttonPressed = false;

    
}


void MyCDCBasicTasks()
{
     unsigned short X1,X2;
     uint16_t i,j,k;
     uint8_t numBytesRead;
     unsigned char FIN[5][20];
     numBytesRead = getsUSBUSART(readBuffer, sizeof(readBuffer));

    if( USBUSARTIsTxTrfReady() == true)
    {

        if(numBytesRead > 0)
        {
            j=0;k=0;
            for(i=0;i<numBytesRead;i++)
            {
                if(readBuffer[i] == '#')
                {
                    FIN[j][k]=0;
                    j++;
                    if(j>5){j=4;}
                    k=0;
                }
                else
                {
                    FIN[j][k]=readBuffer[i];
                    k++;if(k>20){k=19;}
                }
            }
            int TEST=1;
            if(FIN[0][0]=='A' && FIN[0][1]=='A' && FIN[0][2]==0)//記録データ送信要求の場合
            {
                sprintf(writeBuffer,"M0=%d\r\ngN=%d\r\ngSNo=%d\r\nVth=%d\r\n",M0,gN,gSNo,gVth);
                numBytesRead = strlen(writeBuffer);
                putUSBUSART(writeBuffer,numBytesRead);
                fOut=1;gi=0;//記録データ送信フラグON
            }
            if(FIN[0][0]=='B' && FIN[0][1]=='B' && FIN[0][2]==0)//モードと記録間隔の設定
            {
                X1=atoi((const char *)FIN[1]);
                X2=atoi((const char *)FIN[2]);

                Write_EPROM(6,(unsigned char)X1);
                __delay_ms( 10 );
                Write_EPROM(2,(unsigned char)(X2/256));
                __delay_ms( 10 );
                Write_EPROM(3,(unsigned char)(X2));
                __delay_ms( 10 );
                M0=X1;gN=X2;
            }
            if(FIN[0][0]=='C' && FIN[0][1]=='C' && FIN[0][2]==0)//重量変化閾値設定
            {
                X1=atoi((const char *)FIN[1]);
                Write_EPROM(7,(unsigned char)X1);
                gVth=X1;
            }
        }
        else
        {
             if(fOut==1)//記録データ送信フラグONの場合
             {
                 if(gi < gSNo)//記録データ数分を送信
                 {
                     X1=256*Read_EPROM(4*gi+8)+Read_EPROM(4*gi+9);
                     X2=256*Read_EPROM(4*gi+10)+Read_EPROM(4*gi+11);
                     sprintf(writeBuffer,"%d\t%d\t%d\r\n",gi,X1,X2);
                     numBytesRead = strlen(writeBuffer);
                     putUSBUSART(writeBuffer,numBytesRead);
                     gi++;
                 }
                 else
                 {
                     fOut=0;//記録データ送信完了
                 }
             }
        }
    }
     
    CDCTxService();
}



unsigned short ADC0()
{
    unsigned short result;
    long X;

    ADCON0bits.GO = 0;              // Stop AD conversion
    __delay_us( 100 );
    ADCON0bits.CHS = 0b0000; //Analog Channel Select bits=Channel 0 (AN0)
    __delay_us( 100 );
    ADCON0bits.GO = 1;              // Start AD conversion
    while(ADCON0bits.NOT_DONE);     // Wait for conversion
    result=ADRESH*256+ADRESL;
    return result;
}

unsigned short ADC1()
{
    unsigned short result;
    long X;

    ADCON0bits.GO = 0;              // Stop AD conversion
    __delay_us( 100 );
    ADCON0bits.CHS = 0b0001; //Analog Channel Select bits=Channel 0 (AN0)
    __delay_us( 100 );
    ADCON0bits.GO = 1;              // Start AD conversion
    while(ADCON0bits.NOT_DONE);     // Wait for conversion
    result=ADRESH*256+ADRESL;
    return result;
}

//１バイト書込み
void Write_EPROM(unsigned short addr,unsigned char send_data)
{
    TRISB   =	0b00000011; //B1=SCL(INPUT),BO=SDA(OUTPUT)I2C通信
    SSPCON1 = 0x28;		// SSP enable & I2C Master mode, clock = FOSC / (4 * (SSPADD+1))
    SSPADD = 0x0F;		// 0x00だと動作しないクロック設定

    unsigned char    addr1, addr2;
    addr2=addr;
    addr1=addr >> 8;

     //１バイト書込み
        PIR1bits.SSPIF = 0;//I2C_通信割込みフラグクリア
	SSPCON2bits.SEN = 1;//I2C_startビット
	while (SSPCON2bits.SEN) {}//終了待ち

	PIR1bits.SSPIF = 0;//I2C_通信割込みフラグクリア
        while(SSPSTATbits.BF){};
	SSPBUF =0xA0;//0xA0 //書込み制御コード
        if (SSPCON2bits.ACKSTAT) {PORTD=0b11111110;}//ACKチェック
	while (!PIR1bits.SSPIF) {}//終了待ち

	PIR1bits.SSPIF = 0;//I2C_通信割込みフラグクリア
        while(SSPSTATbits.BF){};//SSPBUFの干渉確認
	SSPBUF = addr1; // 書き込みアドレス1
        if (SSPCON2bits.ACKSTAT) {PORTD=0b11111101;}//ACKチェック
	while (!PIR1bits.SSPIF) {}//終了待ち

	PIR1bits.SSPIF = 0;//I2C_通信割込みフラグクリア
        while(SSPSTATbits.BF){};//SSPBUFの干渉確認
	SSPBUF = addr2;// 書き込みアドレス2
        if (SSPCON2bits.ACKSTAT) {PORTD=0b11111100;}//ACKチェック
	while (!PIR1bits.SSPIF) {}//終了待ち

	PIR1bits.SSPIF = 0;//I2C_通信割込みフラグクリア
        while(SSPSTATbits.BF){};//SSPBUFの干渉確認
	SSPBUF =send_data;// send_data書き込み
        if (SSPCON2bits.ACKSTAT) {PORTD=0b11111011;}//ACKチェック
	while (!PIR1bits.SSPIF) {}//終了待ち

	PIR1bits.SSPIF = 0;//I2C_通信割込みフラグクリア
	SSPCON2bits.PEN = 1;//I2C_通信終了
	while (SSPCON2bits.PEN) {}//終了待ち

}

//N個のgEPdata[i]を同時にEPROMに書込み
void Write_EPROM_N(unsigned short addr,unsigned char N)
{
    int i;

    TRISB   =	0b00000011; //B1=SCL(INPUT),BO=SDA(OUTPUT)I2C通信
    SSPCON1 = 0x28;		// SSP enable & I2C Master mode, clock = FOSC / (4 * (SSPADD+1))
    SSPADD = 0x0F;		// 0x00だと動作しないクロック設定

    unsigned char    addr1, addr2;
    addr2=addr;
    addr1=addr >> 8;

     //１バイト書込み
        PIR1bits.SSPIF = 0;//I2C_通信割込みフラグクリア
	SSPCON2bits.SEN = 1;//I2C_startビット
	while (SSPCON2bits.SEN) {}//終了待ち

	PIR1bits.SSPIF = 0;//I2C_通信割込みフラグクリア
        while(SSPSTATbits.BF){};
	SSPBUF =0xA0;//0xA0 //書込み制御コード
        if (SSPCON2bits.ACKSTAT) {PORTD=0b11111110;}//ACKチェック
	while (!PIR1bits.SSPIF) {}//終了待ち

	PIR1bits.SSPIF = 0;//I2C_通信割込みフラグクリア
        while(SSPSTATbits.BF){};//SSPBUFの干渉確認
	SSPBUF = addr1; // 書き込みアドレス1
        if (SSPCON2bits.ACKSTAT) {PORTD=0b11111101;}//ACKチェック
	while (!PIR1bits.SSPIF) {}//終了待ち

	PIR1bits.SSPIF = 0;//I2C_通信割込みフラグクリア
        while(SSPSTATbits.BF){};//SSPBUFの干渉確認
	SSPBUF = addr2;// 書き込みアドレス2
        if (SSPCON2bits.ACKSTAT) {PORTD=0b11111100;}//ACKチェック
	while (!PIR1bits.SSPIF) {}//終了待ち

        for(i=0;i<N;i++)
        {
            PIR1bits.SSPIF = 0;//I2C_通信割込みフラグクリア
            while(SSPSTATbits.BF){};//SSPBUFの干渉確認
            SSPBUF =gEPdata[i];// send_data書き込み
            if (SSPCON2bits.ACKSTAT) {PORTD=0b11111011;}//ACKチェック
            while (!PIR1bits.SSPIF) {}//終了待ち
        }
        PIR1bits.SSPIF = 0;//I2C_通信割込みフラグクリア
        SSPCON2bits.PEN = 1;//I2C_通信終了
        while (SSPCON2bits.PEN) {}//終了待ち
}

//１バイト読込
unsigned char   Read_EPROM(unsigned short addr)
{
    TRISB   =	0b00000011; //B1=SCL(INPUT),BO=SDA(OUTPUT)I2C通信
    SSPCON1 = 0x28;		// SSP enable & I2C Master mode, clock = FOSC / (4 * (SSPADD+1))
    SSPADD = 0x0F;		// 0x00だと動作しないクロック設定

    unsigned char    addr1, addr2;
    unsigned char data=5;


    addr2=addr;
    addr1=addr >> 8;

    //１バイト読込
        PIR1bits.SSPIF = 0;//I2C_通信割込みフラグクリア
	SSPCON2bits.SEN = 1;//I2C_startビット
	while (SSPCON2bits.SEN) {}

	PIR1bits.SSPIF = 0;//I2C_通信割込みフラグクリア
        while(SSPSTATbits.BF){};//SSPBUFの干渉確認
	SSPBUF =0xA0;//0xA0//書込み制御コード
        if (SSPCON2bits.ACKSTAT) {PORTD=0b11111010;}//ACKチェック
	while (!PIR1bits.SSPIF) {}//終了待ち

	PIR1bits.SSPIF = 0;//I2C_通信割込みフラグクリア
        while(SSPSTATbits.BF){};//SSPBUFの干渉確認
	SSPBUF = addr1;// 書き込みアドレス1
        if (SSPCON2bits.ACKSTAT) {PORTD=0b11111001;}//ACKチェック
	while (!PIR1bits.SSPIF) {}//終了待ち

	PIR1bits.SSPIF = 0;//I2C_通信割込みフラグクリア
        while(SSPSTATbits.BF){};//SSPBUFの干渉確認
	SSPBUF = addr2; // 書き込みアドレス2
        if (SSPCON2bits.ACKSTAT) {PORTD=0b11111000;}//ACKチェック
	while (!PIR1bits.SSPIF) {}//終了待ち

	SSPCON2bits.SEN = 1;//I2C_startビット
	while (SSPCON2bits.SEN) {}//終了待ち

	PIR1bits.SSPIF = 0;//I2C_通信割込みフラグクリア
        while(SSPSTATbits.BF){};//SSPBUFの干渉確認
	SSPBUF =  0xA1; //0xA1 //読込制御コード
        if (SSPCON2bits.ACKSTAT) {PORTD=0b11110111;}//ACKチェック
	while (!PIR1bits.SSPIF) {}//終了待ち

	PIR1bits.SSPIF = 0;//I2C_通信割込みフラグクリア
	SSPCON2bits.RCEN = 1;//受信モードを動作
	while (SSPCON2bits.RCEN) {}//終了待ち

	data=SSPBUF;//受信コピー

        PIR1bits.SSPIF = 0;//I2C_通信割込みフラグクリア
        SSPCON2bits.ACKDT = 1;//受信の終わりで応答シーケンスを開始1=NO-ACK
        SSPCON2bits.ACKEN = 1;//、ACKDT データビットを送信する。
	while (SSPCON2bits.ACKEN) {}//終了待ち

	PIR1bits.SSPIF = 0;//I2C_通信割込みフラグクリア
	SSPCON2bits.PEN = 1;//I2C_通信終了
	while (SSPCON2bits.PEN) {}//終了待ち

    return data;
}